"""Socrata API settings and constants"""

# Request timeout in seconds
REQUEST_TIMEOUT = 30

# Maximum number of records to fetch per page
# Socrata API supports up to 50000 records per request
DEFAULT_PAGE_SIZE = 50000
